@@ -27,6 +27,7 @@ import requests
 from deprecation import deprecated
 from logzero import setup_logger
 from PIL import Image
+from retry import retry
 
 from . import bplist
 from ._crash import CrashManager
@@ -786,6 +787,8 @@ class BaseDevice():
             conn = self.start_service(LockdownService.TestmanagerdLockdown)
         return DTXService(conn)
 
+    # 2022-08-24 add retry delay, looks like sometime can recover
+    @retry((ssl.SSLError, socket.timeout), delay=10, jitter=1, tries=3, logger=logging)
     def connect_instruments(self) -> ServiceInstruments:
         """ start service for instruments """
         if self.major_version() >= 14:
