@@ -29,7 +29,7 @@ from retry import retry
 from . import bplist
 from . import struct2 as ct
 from ._proto import LOG, InstrumentsService
-from ._safe_socket import PlistSocketProperty
+from ._safe_socket import PlistSocketProxy
 from .exceptions import MuxError, ServiceError
 
 logger = logging.getLogger(LOG.xctest)
@@ -267,7 +267,7 @@ class AUXMessageBuffer(object):
     #     else:
     #         self.append_obj(v)
 
-class DTXService(PlistSocketProperty):
+class DTXService(PlistSocketProxy):
 
     def prepare(self):
         super().prepare()
@@ -288,7 +288,9 @@ class DTXService(PlistSocketProperty):
         }
         payload = DTXPayload.build('_notifyOfPublishedCapabilities:', [capabilities])
         self.send_dtx_message(channel=0, payload=payload)
+        self._dtx_message_pool = {}
         self._drain_background()  # 开启接收线程
+        weakref.finalize(self, self._stop_event.set)
 
     def _next_message_id(self) -> int:
         self._last_message_id += 1
@@ -396,6 +398,39 @@ class DTXService(PlistSocketProperty):
         self.psock.sendall(data)
         return _message_id
 
+    def recv_part_dtx_message(self) -> typing.Optional[int]:
+        """
+        DTXMessage contains one or more fragmenets
+        This fragments may received in different orders
+
+        Returns:
+            None or message_id
+        """
+        data = self.psock.recvall(0x20)
+        h = DTXMessageHeader.parse(data)
+        if h.magic != 0x1F3D5B79:
+            raise MuxError("bad header magic: 0x%x\n" % h.magic)
+
+        if h.header_length != 0x20:
+            raise MuxError("header length expect 0x20, got 0x%x".format(
+                h.header_length))
+        
+        if h.fragment_id == 0:
+            # when reading multiple message fragments
+            # only the 0th fragment contains a message header
+            # but the 0th payload is empty
+            self._dtx_message_pool[h.message_id] = (h, bytearray())
+            if h.fragment_count > 1:
+                return None
+        _, payload = self._dtx_message_pool[h.message_id]
+        rawdata = self.psock.recvall(h.payload_length)
+        payload.extend(rawdata)
+        
+        if h.fragment_id == h.fragment_count - 1:
+            return h.message_id
+        else:
+            return None
+
     def recv_dtx_message(self) -> Tuple[Any, bytearray]:
         """
         前32个字节(两行) 为DTXMessage的头部 (包含了消息的类型和请求的channel)
@@ -474,37 +509,14 @@ class DTXService(PlistSocketProperty):
         00000010: 03 00 00 00 01 00 00 00  00 00 00 00 00 00 00 00  ................
         00000020: 00 00 00 00 00 00 00 00  00 00 00 00 00 00 00 00  ................
         """
-        payload = bytearray()
-        mheader = None
-
         while True:
-            data = self.psock.recvall(0x20)
-            h = DTXMessageHeader.parse(data)
-            # print("frag:", fragment_id, fragment_count)
-            if h.magic != 0x1F3D5B79:
-                raise MuxError("bad header magic: 0x%x\n" % h.magic)
-
-            if h.header_length != 0x20:
-                raise MuxError("header length expect 0x20, got 0x%x".format(
-                    h.header_length))
-
-            if h.fragment_id == 0:
-                # when reading multiple message fragments
-                # only the 0th fragment contains a message header
-                # but the 0th payload is empty
-                mheader = h
-                if h.fragment_count > 1:
-                    continue
-
-            rawdata = self.psock.recvall(h.payload_length)
-            payload.extend(rawdata)
-
-            if h.fragment_id == h.fragment_count - 1:
-                break
-
-        assert mheader.conversation_index in [0, 1, 2]
-        assert mheader.expects_reply in [0, 1]
-        return (mheader, payload)
+            message_id = self.recv_part_dtx_message()
+            if message_id is None:
+                continue
+            mheader, payload = self._dtx_message_pool.pop(message_id)
+            assert mheader.conversation_index in [0, 1, 2]
+            assert mheader.expects_reply in [0, 1]
+            return (mheader, payload)
 
     def _call_handlers(self, event_name: Event, data: Any = None) -> bool:
         """
@@ -523,7 +535,6 @@ class DTXService(PlistSocketProperty):
         return True
 
     def _handle_dtx_message(self, m: DTXMessage) -> bool:
-        # logger.warning("Callback: identifier: %s", m.result) # TODO
         assert m.header.expects_reply == 1
 
         if m.channel_id == 0xFFFFFFFF and m.flags == 0x05:
@@ -538,7 +549,6 @@ class DTXService(PlistSocketProperty):
             #     logger.debug("logDebugMessage: %s", args)
             #     self._reply_null(m)
 
-            # logger.warning("Callback: identifier: %s", m.result) # TODO
             if self._call_handlers(identifier, m):
                 return True
         # else:
@@ -547,12 +557,18 @@ class DTXService(PlistSocketProperty):
 
     def wait_reply(self, message_id: int, timeout=30.0) -> DTXMessage:
         """
+        Raises:
+            MuxError, ServiceError
+
         Refs: https://www.tornadoweb.org/en/stable/guide/async.html#asynchronous
         """
-        ret = self._reply_queues[message_id].get(timeout=timeout)
-        if ret is None:
-            raise MuxError("connection closed")
-        return ret
+        try:
+            ret = self._reply_queues[message_id].get(timeout=timeout)
+            if ret is None:
+                raise MuxError("connection closed")
+            return ret
+        except queue.Empty:
+            raise ServiceError("wait reply timeout")
 
     def _drain_background(self):
         threading.Thread(name="DTXMessage", target=self._drain, daemon=True).start()
@@ -565,6 +581,10 @@ class DTXService(PlistSocketProperty):
                 except MuxError as e:
                     # logger.warning("unexpected error: %s", e)
                     break
+                except OSError:
+                    if self._stop_event.is_set():
+                        break
+                    raise
                 except:
                     if not self._stop_event.is_set():
                         raise
@@ -634,10 +654,10 @@ class ServiceInstruments(DTXService):
 
     def prepare(self):
         super().prepare()
-        self._finalizer = weakref.finalize(self, self.close)
 
     def app_launch(self,
                    bundle_id: str,
+                   app_env: dict = {},
                    args: list = [],
                    kill_running: bool = False) -> int:
         """
@@ -647,8 +667,6 @@ class ServiceInstruments(DTXService):
         code = self.make_channel(self._SERVICE_PROCESS_CONTROL)
         method = "launchSuspendedProcessWithDevicePath:bundleIdentifier:environment:arguments:options:"
         app_path = ""  # not used, just pass empty string
-        app_env = {
-        }  # environment variables: not used, just pass empty dictionary
         #app_args = []  # not used, just pass empty array
 
         options = {
@@ -672,7 +690,7 @@ class ServiceInstruments(DTXService):
 
         self.call_message(channel, "killPid:", [pid], expects_reply=False)
 
-    def app_running_processes(self):
+    def app_running_processes(self) -> typing.List[dict]:
         """
         Returns array of dict:
             {'isApplication': False,
@@ -707,7 +725,9 @@ class ServiceInstruments(DTXService):
         """
         def exefile2appinfo(exe_abspath: str, app_infos: List[dict]):
             for info in app_infos:
-                path = info['Path'] + "/" + info['CFBundleExecutable']
+                # info may not contain key "Path"
+                # https://github.com/alibaba/taobao-iphone-device/issues/61
+                path = info.get('Path', "") + "/" + info['CFBundleExecutable']
                 if path.startswith("/private"):
                     path = path[len("/private"):]
                 if exe_abspath == path:
@@ -919,7 +939,44 @@ class ServiceInstruments(DTXService):
             # aux = AUXMessageBuffer()
             # aux.append_obj(channel_id)
             # self.send_dtx_message(channel_id, DTXPayload.build('_channelCanceled:', aux))
-            
+
+    def start_energy_sampling(self, pid: int):
+        ch_network = InstrumentsService.XcodeEnergyStatistics
+        return self.call_message(ch_network, 'startSamplingForPIDs:', [{pid}])
+
+    def stop_energy_sampling(self, pid: int):
+        ch_network = InstrumentsService.XcodeEnergyStatistics
+        return self.call_message(ch_network, 'stopSamplingForPIDs:', [{pid}])
+
+    def get_process_energy_stats(self, pid: int) -> Optional[dict]:
+        """
+        Returns dict:
+            example:
+            {
+                "energy.overhead": -10,
+                "kIDEGaugeSecondsSinceInitialQueryKey": 10,
+                "energy.version": 1,
+                "energy.gpu.cost": 0,
+                "energy.cpu.cost": 0.10964105931592821,
+                "energy.networkning.overhead": 0,
+                "energy.appstate.cost": 8,
+                "energy.location.overhead": 0,
+                "energy.thermalstate.cost": 0,
+                "energy.networking.cost": 0,
+                "energy.cost": -9.890358940684072,
+                "energy.display.cost": 0,
+                "energy.cpu.overhead": 0,
+                "energy.location.cost": 0,
+                "energy.gpu.overhead": 0,
+                "energy.appstate.overhead": 0,
+                "energy.display.overhead": 0,
+                "energy.inducedthermalstate.cost": -1
+            }
+        """
+        ch_network = InstrumentsService.XcodeEnergyStatistics
+        args = [{}, {pid}]
+        ret = self.call_message(ch_network, 'sampleAttributes:forPIDs:', args)
+        return ret.get(pid)
 
     def start_network_sampling(self, pid: int):
         ch_network = 'com.apple.xcode.debug-gauge-data-providers.NetworkStatistics'
@@ -929,7 +986,7 @@ class ServiceInstruments(DTXService):
         ch_network = 'com.apple.xcode.debug-gauge-data-providers.NetworkStatistics'
         return self.call_message(ch_network, 'stopSamplingForPIDs:', [{pid}])
 
-    def get_process_network_stats(self, pid: int) -> Optional[Iterator[dict]]:
+    def get_process_network_stats(self, pid: int) -> Optional[dict]:
         """
         经测试数据始终不是很准，用safari测试，每次刷新图片的时候，rx.bytes总是不动
         """
